@@ -3,15 +3,24 @@
 State tracking functionality for django models
 """
 import inspect
-from functools import wraps
 import sys
+import warnings
+from functools import wraps
 
 from django.db import models
-from django.db.models.loading import get_model
 from django.db.models.signals import class_prepared
 from django.utils.functional import curry
 from django_fsm.signals import pre_transition, post_transition
 
+try:
+    from django.apps import apps as django_apps
+
+    def get_model(app_label, model_name):
+        app = django_apps.get_app_config(app_label)
+        return app.get_model(model_name)
+except ImportError:
+    from django.db.models.loading import get_model
+
 
 __all__ = ['TransitionNotAllowed', 'ConcurrentTransition',
            'FSMFieldMixin', 'FSMField', 'FSMIntegerField',
@@ -50,6 +59,11 @@ else:
 class TransitionNotAllowed(Exception):
     """Raised when a transition is not allowed"""
 
+    def __init__(self, *args, **kwargs):
+        self.object = kwargs.pop('object', None)
+        self.method = kwargs.pop('method', None)
+        super(TransitionNotAllowed, self).__init__(*args, **kwargs)
+
 
 class ConcurrentTransition(Exception):
     """
@@ -73,10 +87,16 @@ class Transition(object):
     def name(self):
         return self.method.__name__
 
-    def has_perm(self, user):
+    def has_perm(self, instance, user):
         if not self.permission:
             return True
-        elif callable(self.permission) and self.permission(user):
+        elif callable(self.permission):
+            try:
+                return bool(self.permission(instance, user))
+            except TypeError:
+                warnings.warn('Callable permissions without instance parameter is depricated')
+                return bool(self.permission(user))
+        elif user.has_perm(self.permission, instance):
             return True
         elif user.has_perm(self.permission):
             return True
@@ -95,7 +115,7 @@ def get_available_FIELD_transitions(instance, field):
     for name, transition in transitions.items():
         meta = transition._django_fsm
 
-        for state in [curr_state, '*']:
+        for state in [curr_state, '*', '+']:
             if state in meta.transitions:
                 transition = meta.transitions[state]
                 if all(map(lambda condition: condition(instance), transition.conditions)):
@@ -115,7 +135,7 @@ def get_available_user_FIELD_transitions(instance, user, field):
     with all conditions met and user have rights on it
     """
     for transition in get_available_FIELD_transitions(instance, field):
-        if transition.has_perm(user):
+        if transition.has_perm(instance, user):
             yield transition
 
 
@@ -131,6 +151,8 @@ class FSMMeta(object):
         transition = self.transitions.get(source, None)
         if transition is None:
             transition = self.transitions.get('*', None)
+        if transition is None:
+            transition = self.transitions.get('+', None)
         return transition
 
     def add_transition(self, method, source, target, on_error=None, conditions=[], permission=None, custom={}):
@@ -150,7 +172,16 @@ class FSMMeta(object):
         """
         Lookup if any transition exists from current model state using current method
         """
-        return state in self.transitions or '*' in self.transitions
+        if state in self.transitions:
+            return True
+
+        if '*' in self.transitions:
+            return True
+
+        if '+' in self.transitions and self.transitions['+'].target != state:
+            return True
+
+        return False
 
     def conditions_met(self, instance, state):
         """
@@ -171,7 +202,7 @@ class FSMMeta(object):
         if not transition:
             return False
         else:
-            return transition.has_perm(user)
+            return transition.has_perm(instance, user)
 
     def next_state(self, current_state):
         transition = self.get_transition(current_state)
@@ -267,9 +298,14 @@ class FSMFieldMixin(object):
         method_name = method.__name__
         current_state = self.get_state(instance)
 
-        if not (meta.has_transition(current_state) and meta.conditions_met(instance, current_state)):
+        if not meta.has_transition(current_state):
             raise TransitionNotAllowed(
-                "Can't switch from state '{0}' using method '{1}'".format(current_state, method_name))
+                "Can't switch from state '{0}' using method '{1}'".format(current_state, method_name),
+                object=instance, method=method)
+        if not meta.conditions_met(instance, current_state):
+            raise TransitionNotAllowed(
+                "Transition conditions have not been met for method '{0}'".format(method_name),
+                object=instance, method=method)
 
         next_state = meta.next_state(current_state)
 
@@ -285,7 +321,7 @@ class FSMFieldMixin(object):
 
         try:
             result = method(instance, *args, **kwargs)
-            if next_state:
+            if next_state is not None:
                 self.set_proxy(instance, next_state)
                 self.set_state(instance, next_state)
         except Exception as exc:
@@ -295,8 +331,9 @@ class FSMFieldMixin(object):
                 self.set_state(instance, exception_state)
                 signal_kwargs['target'] = exception_state
                 signal_kwargs['exception'] = exc
+                post_transition.send(**signal_kwargs)
             raise
-        finally:
+        else:
             post_transition.send(**signal_kwargs)
 
         return result
@@ -458,22 +495,26 @@ def transition(field, source='*', target=None, on_error=None, conditions=[], per
     has not changed after the function call
     """
     def inner_transition(func):
-        fsm_meta = getattr(func, '_django_fsm', None)
+        wrapper_installed, fsm_meta = True, getattr(func, '_django_fsm', None)
         if not fsm_meta:
+            wrapper_installed = False
             fsm_meta = FSMMeta(field=field, method=func)
             setattr(func, '_django_fsm', fsm_meta)
 
-        @wraps(func)
-        def _change_state(instance, *args, **kwargs):
-            return fsm_meta.field.change_state(instance, func, *args, **kwargs)
-
         if isinstance(source, (list, tuple)):
             for state in source:
                 func._django_fsm.add_transition(func, state, target, on_error, conditions, permission, custom)
         else:
             func._django_fsm.add_transition(func, source, target, on_error, conditions, permission, custom)
 
-        return _change_state
+        @wraps(func)
+        def _change_state(instance, *args, **kwargs):
+            return fsm_meta.field.change_state(instance, func, *args, **kwargs)
+
+        if not wrapper_installed:
+            return _change_state
+
+        return func
 
     return inner_transition
 
