@@ -1,5 +1,5 @@
 PROJECT_APPS = ('django_fsm', 'testapp',)
-INSTALLED_APPS = ('django.contrib.contenttypes', 'django.contrib.auth', 'django_jenkins',) + PROJECT_APPS
+INSTALLED_APPS = ('django.contrib.contenttypes', 'django.contrib.auth', 'guardian', 'django_jenkins') + PROJECT_APPS
 DATABASE_ENGINE = 'sqlite3'
 SECRET_KEY = 'nokey'
 MIDDLEWARE_CLASSES = ()
@@ -13,3 +13,5 @@ JENKINS_TASKS = (
     'django_jenkins.tasks.run_pep8',
     'django_jenkins.tasks.run_pyflakes'
 )
+
+ANONYMOUS_USER_ID=0
